#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>

#include "enc28j60.h"
#include "handler.h"
#include "LCD.h"

/*
 * PB0 -> SI
 * PB1 <- SO
 * PB2 <- INT
 * PB3 -> SCK
 * PB4 -> CS
 *
 * PD0 -> RESET
 */
#define TEST_LED		PD6
#define TEST_LED_PORT	PORTD
#define TEST_LED_DDR	DDRD

#define ENC_GREEN_LED_SHIFT		4
#define ENC_YELLOW_LED_SHIFT	8

/*
0000 = Reserved
0001 = Display transmit activity (stretchable)
0010 = Display receive activity (stretchable)
0011 = Display collision activity (stretchable)
0100 = Display link status
0101 = Display duplex status
0110 = Reserved
0111 = Display transmit and receive activity (stretchable)
1000 = On
1001 = Off
1010 = Blink fast
1011 = Blink slow
1100 = Display link status and receive activity (always stretched)
1101 = Display link status and transmit/receive activity (always stretched)
1110 = Display duplex status and collision activity (always stretched)
1111 = Reserved
*/
#define ENC_DISPLAY_TRANSMIT_ACTIVITY		1
#define ENC_DISPLAY_RECEIVE_ACTIVITY		2
#define ENC_DISPLAY_COLLISION_ACTIVITY		3
#define ENC_DISPLAY_LINK_STATUS				4
#define ENC_DISPLAY_DUPLEX_STATUS			5
#define ENC_DISPLAY_TRANSMIT_AND_RECEIVE	7
#define ENC_DISPLAY_ON						8
#define ENC_DISPLAY_OFF						9
#define ENC_DISPLAY_BLINKFAST				10
#define ENC_DISPLAY_BLINKSLOW				11
#define ENC_DISPLAY_LINK_AND_RECEIVE		12
#define ENC_DISPLAY_LINK_TRANSMIT_RECEIVE	13
#define	ENC_DISPLAY_DUPLEX_AND_COLLISION	14

uint8_t count;

ISR(INT2_vect)
{
	while (!(ENC28J60_SPI_PIN & (1 << ENC28J60_SPI_SS)))
		InterruptHandler();
}
ISR(TIMER0_OVF_vect)
{
 	TEST_LED_PORT ^= (1 << TEST_LED); 
	if (++count >= 9)
	{
		count = 0;

		//QueryMAC(MakeIP(192, 168, 0, 110));
	}
}
void InitTimer()
{
	TCCR0 |= (1 << CS02) | (1 << CS00);
	TIMSK |= (1 << TOIE0);
}
int main()
{
	lcd_init();
	lcd_string("LCD Loaded");
	myip[0] = 192;
    myip[1] = 168;
    myip[2] = 0;
    myip[3] = 150;
	
	mymac[0] = 0x00;
    mymac[1] = 0x18;
    mymac[2] = 0xAA;
    mymac[3] = 0xb1;
    mymac[4] = 0x15;
    mymac[5] = 0x5f;

	ENC28J60_RESET_DDR |= (1 << ENC28J60_RESET);
	ENC28J60_RESET_PORT &= ~(1 << ENC28J60_RESET);
	_delay_ms(1);
	ENC28J60_RESET_PORT |= (1 << ENC28J60_RESET);

	enc28j60Init();
	InitTimer();

	GICR |= (1 << INT2);
	
	enc28j60PhyWrite(PHLCON, (ENC_DISPLAY_LINK_STATUS << ENC_YELLOW_LED_SHIFT) | (ENC_DISPLAY_TRANSMIT_AND_RECEIVE << ENC_GREEN_LED_SHIFT));
 
 	TEST_LED_DDR |= (1 << TEST_LED); 

	sei(); 

	set_sleep_mode(SLEEP_MODE_IDLE);
	for (;;)
    	sleep_mode();
}
