#include <avr/io.h>

#include "handler.h"
#include "enc28j60.h"
#include "LCD.h"

uint8_t ethernet_buffer[ETHERNET_MTU];

void movemem(void *v_dest, void *v_src, uint16_t size)
{
	uint8_t *dest = (uint8_t *)v_dest;
	uint8_t *src = (uint8_t *)v_src;

	if (dest > src)
	{
		dest += size;
		src += size;
		for (uint16_t i = 0; i < size; ++i)
			*--dest = *--src;
	}
	else
	{
		for (uint16_t i = 0; i < size; ++i)
			*dest++ = *src++;
	}
}
uint8_t equalmem(void *ptr1, void *ptr2, uint16_t size)
{
	uint8_t *src1 = (uint8_t *)ptr1;
	uint8_t *src2 = (uint8_t *)ptr2;

	for (uint16_t i = 0; i < size; ++i)
	{
		if (*src1++ != *src2++)
			return 0;
	}
	return 1;
}
int checksum (uint8_t *pointer, uint32_t result32, uint16_t result16)
{
	uint16_t result16_1 = 0x0000;
	uint8_t DataH;
	uint8_t DataL;
	
	//Jetzt werden alle Packete in einer While Schleife addiert
	while (result16 > 1)
	{
		//schreibt Inhalt Pointer nach DATAH danach inc Pointer
		DataH = *pointer++;

		//schreibt Inhalt Pointer nach DATAL danach inc Pointer
		DataL = *pointer++;

		//erzeugt Int aus Data L und Data H
		result16_1 = ((DataH << 8)+DataL);
		//Addiert packet mit vorherigen
		result32 = result32 + result16_1;
		//decrimiert Lnge von TCP Headerschleife um 2
		result16 -=2;
	}

	//Ist der Wert result16 ungerade ist DataL = 0
	if (result16 > 0)
	{
		//schreibt Inhalt Pointer nach DATAH danach inc Pointer
		DataH = *pointer;
		//erzeugt Int aus Data L ist 0 (ist nicht in der Berechnung) und Data H
		result16_1 = (DataH << 8);
		//Addiert packet mit vorherigen
		result32 = result32 + result16_1;
	}
	
	//Komplementbildung (addiert Long INT_H Byte mit Long INT L Byte)
	result32 = ((result32 & 0x0000FFFF)+ ((result32 & 0xFFFF0000) >> 16));
	result32 = ((result32 & 0x0000FFFF)+ ((result32 & 0xFFFF0000) >> 16));	
	result16 =~(result32 & 0x0000FFFF);
	
	return result16;
}

void BuildEthernetFrame(uint8_t *buffer, uint8_t *dest_addr)
{
	struct Ethernet_Header *ether = (struct Ethernet_Header *)ethernet_buffer;
	movemem(ether->EnetPacketDest, dest_addr, 6);
	movemem(ether->EnetPacketSrc, mymac, 6);
}
void BuildARPReplyFrame(uint8_t *buffer, uint8_t *dest_addr, uint8_t *dest_ip)
{
	struct Ethernet_Header *ether = (struct Ethernet_Header *)buffer;
	ether->EnetPacketType = 0x0608;

	struct ARP_Header *arp = (struct ARP_Header *)(buffer + ARP_OFFSET);
	arp->ARP_HWType = 0x0100; // ethernet
	arp->ARP_PRType = 0x0008; // link layer ip
	arp->ARP_HWLen = 6; // sizeof mac
	arp->ARP_PRLen = 4; // sizeof ip
	arp->ARP_Op = 0x0200; // answer
	movemem(arp->ARP_SHAddr, mymac, 6);
	movemem(&arp->ARP_SIPAddr, myip, 4);
	movemem(arp->ARP_THAddr, dest_addr, 6);
	movemem(&arp->ARP_TIPAddr, dest_ip, 4);
}
void BuildIpFrame(uint8_t *buffer, uint8_t *dest_addr, uint8_t proto, uint16_t datasize)
{
	struct IP_Header *ip = (struct IP_Header *)(buffer + IP_OFFSET);

	ip->IP_Vers_Len = 0x45;
	ip->IP_Tos = 0;
	ip->IP_Pktlen = sizeof(struct IP_Header) + datasize;

	ip->IP_Id = 0;
	ip->IP_Frag_Offset = 0;

	ip->IP_ttl = 255;
	ip->IP_Proto = proto;
	movemem(&ip->IP_Srcaddr, myip, 4);
	movemem(&ip->IP_Destaddr, dest_addr, 4);

	ip->IP_Hdr_Cksum = 0x00; // checksum auf null setzen
	uint16_t sum = checksum((uint8_t *)ip, 0x00000000, 20); // checksumme ausrechnen
	
	ip->IP_Hdr_Cksum = ((sum & 0xFF00) >> 8) | ((sum & 0x00FF)<<8); 
}
void BuildEchoReply(uint8_t *buffer, uint16_t offset)
{
	struct ICMP_Header *icmp = (struct ICMP_Header *)(buffer + offset);

	icmp->ICMP_Type = 0x00;
	icmp->ICMP_Code = 0x00;
	icmp->ICMP_Cksum = 0x00;

	uint16_t sum = checksum((uint8_t *)icmp, 0x00000000, 4); // checksumme ausrechnen
	
	icmp->ICMP_Cksum = ((sum & 0xFF00) >> 8) | ((sum & 0x00FF) << 8); 
}
void lcd_hexchar(uint8_t val)
{
	if (val <= 9)
		lcd_data(val + '0');
	else
		lcd_data(val - 10 + 'a');
}
void lcd_hex_byte(uint8_t val)
{
	lcd_hexchar((val >> 4) & 0xf);
	lcd_hexchar(val & 0xf);
}
void lcd_hex_short(uint16_t val)
{
	lcd_hex_byte((val >> 8) & 0xff);
	lcd_hex_byte(val & 0xff);
}
void lcd_dec(uint8_t val)
{
	if (val > 100)
	{
		lcd_hexchar(val / 100);
		val %= 100;

		lcd_hexchar(val / 10);
		val %= 10;

		lcd_hexchar(val);
	}
	else if (val > 10)
	{
		lcd_hexchar(val / 10);
		val %= 10;

		lcd_hexchar(val);
	}
	else
	{
		lcd_hexchar(val);
	}
}
void lcd_print_ip(uint8_t *ip)
{
	for (uint8_t i = 0; i < 4; ++i)
	{
		lcd_dec(*ip++);
		if (i != 3)
			lcd_data('.');
	}
}
void lcd_print_mac(uint8_t *mac)
{
	for (uint8_t i = 0; i < 6; ++i)
		lcd_hex_byte(*mac++);
}
void QueryMAC(uint32_t ip)
{
	uint8_t broadcast_addr[6];
	for (uint8_t i = 0; i < 6; ++i)
		broadcast_addr[i] = 0xff;

	BuildEthernetFrame(ethernet_buffer, broadcast_addr);

	struct Ethernet_Header *ether = (struct Ethernet_Header *)ethernet_buffer;
	ether->EnetPacketType = 0x0608;

	struct ARP_Header *arp = (struct ARP_Header *)(ethernet_buffer + ARP_OFFSET);
	arp->ARP_HWType = 0x0100; // ethernet
	arp->ARP_PRType = 0x0008; // link layer ip
	arp->ARP_HWLen = 6; // sizeof mac
	arp->ARP_PRLen = 4; // sizeof ip
	arp->ARP_Op = 0x0100; // answer
	movemem(arp->ARP_SHAddr, mymac, 6);
	movemem(&arp->ARP_SIPAddr, myip, 4);
	movemem(arp->ARP_THAddr, broadcast_addr, 6);
	movemem(&arp->ARP_TIPAddr, &ip, 4);

	enc28j60PacketSend(sizeof(struct Ethernet_Header) + sizeof(struct ARP_Header), ethernet_buffer);
}
uint8_t pos = 0;
void InterruptHandler()
{
	unsigned length = enc28j60PacketReceive(sizeof(ethernet_buffer), ethernet_buffer);
	if (length)
	{
		struct Ethernet_Header *ether = (struct Ethernet_Header *)ethernet_buffer;
		if (ether->EnetPacketType == 0x0608) // ARP
		{
			struct ARP_Header *arp = (struct ARP_Header *)(ethernet_buffer + ARP_OFFSET);
			if (pos < 1)
				pos = 1;
			if (arp->ARP_Op == 0x0100)
			{
				if (ethernet_buffer[0] == 0xff && ethernet_buffer[1] == 0xff && ethernet_buffer[2] == 0xff &&
				ethernet_buffer[3] == 0xff && ethernet_buffer[4] == 0xff && ethernet_buffer[5] == 0xff &&
				equalmem(ethernet_buffer + 38, myip, 4)) // request to us
				{
					if (pos < 2)
						pos = 2;
					// ARP Request for us
					uint8_t remote_mac[6];
					uint8_t remote_ip[4];
					movemem(remote_mac, &ether->EnetPacketSrc, 6);
					movemem(remote_ip, &((struct ARP_Header *)(ethernet_buffer + ARP_OFFSET))->ARP_SIPAddr, 4);
					BuildEthernetFrame(ethernet_buffer, remote_mac);
					BuildARPReplyFrame(ethernet_buffer, remote_mac, remote_ip);

					enc28j60PacketSend(sizeof(struct Ethernet_Header) + sizeof(struct ARP_Header), ethernet_buffer);
					
					lcd_clear();
					lcd_setcursor(2, 1);
					lcd_print_ip((uint8_t *)remote_ip);
					lcd_setcursor(0, 2);
					lcd_print_mac(remote_mac);
				}
			}
			else
			{
				if (equalmem(&arp->ARP_TIPAddr, myip, 4) && equalmem(arp->ARP_THAddr, mymac, 6))
				{
					lcd_clear();
					lcd_setcursor(2, 0);
					lcd_print_ip((uint8_t *)&arp->ARP_SIPAddr);
					lcd_setcursor(0, 2);
					lcd_print_mac(arp->ARP_SHAddr);
				}
			}
		}
		/*if (ether->EnetPacketType == 0x0008) // ip
		{
			uint8_t remote_mac[6];
			movemem(remote_mac, &ether->EnetPacketSrc, 6);
			struct IP_Header *ip = (struct IP_Header *)(ethernet_buffer + IP_OFFSET);
			if ((ip->IP_Vers_Len & 0xf0) == 0x40 && (ip->IP_Vers_Len & 0xf) < 15 && equalmem(&ip->IP_Destaddr, myip, 4))
			{
				uint8_t remote_ip[4];
				movemem(remote_ip, &((struct IP_Header *)(ethernet_buffer + IP_OFFSET))->IP_Srcaddr, 4);
				if (pos < 3)
					pos = 3;
				if (ip->IP_Proto == 1) // ICMP
				{
					if (pos < 4)
						pos = 4;

					uint16_t offset = ((ip->IP_Vers_Len & 0xf) - 5) * 4;
					struct ICMP_Header *icmp = (struct ICMP_Header *)(ethernet_buffer + ICMP_OFFSET + offset);
					if (icmp->ICMP_Type == 0x08 && icmp->ICMP_Code == 0x00) // ping request
					{
						if (pos < 5)
							pos = 5;

						BuildEthernetFrame(ethernet_buffer, remote_mac);
						BuildEchoReply(ethernet_buffer, ICMP_OFFSET);
						BuildIpFrame(ethernet_buffer, remote_ip, 1, sizeof(struct ICMP_Header)); // build later to calc checksum

						pos = 6;

						enc28j60PacketSend(sizeof(struct Ethernet_Header) + sizeof(struct IP_Header) + sizeof(struct ICMP_Header), ethernet_buffer);
					}
				}
			}
		}*/		
		lcd_home();
		lcd_hexchar(pos);

	}
}
unsigned long MakeIP(unsigned char byte1, unsigned char byte2, unsigned char byte3, unsigned char byte4)
{
	return (unsigned long)byte1 | ((unsigned long)byte2 << 8) | ((unsigned long)byte3 << 16) | ((unsigned long)byte4 << 24);
}
