#ifndef HANDLER_H
#define HANDLER_H

#define ETHERNET_MTU 1518
extern uint8_t ethernet_buffer[ETHERNET_MTU];

#define ETHER_OFFSET			0x00
#define ARP_OFFSET				0x0E
#define IP_OFFSET				0x0E
#define ICMP_OFFSET				0x22
#define ICMP_DATA				0x2A
#define TCP_OFFSET				0x22

//IP Protocol Types
#define	PROT_ICMP				0x01	//zeigt an die Nutzlasten enthalten das ICMP Prot
#define	PROT_TCP				0x06	//zeigt an die Nutzlasten enthalten das TCP Prot.
#define	PROT_UDP				0x11	//zeigt an die Nutzlasten enthalten das UDP Prot.	

//Defines fr IF Abfrage
#define MYIP 			ip->IP_Destaddr==*((unsigned long*)&myip[0])
#define IP_UDP_PACKET 	ip->IP_Proto == PROT_UDP
#define IP_TCP_PACKET 	ip->IP_Proto == PROT_TCP
#define IP_ICMP_PACKET 	ip->IP_Proto == PROT_ICMP

//Defines fr if Abfrage
#define ETHERNET_IP_DATAGRAMM	ethernet->EnetPacketType == 0x0008
#define ETHERNET_ARP_DATAGRAMM 	ethernet->EnetPacketType == 0x0608

#define IP_VERS_LEN 20
#define TCP_HDR_LEN 20
#define ETH_HDR_LEN 14
#define TCP_DATA_START IP_VERS_LEN+TCP_HDR_LEN+ETH_HDR_LEN

#define MAX_WINDOWS_SIZE 0xFFFF

//TCP Flags
#define  FIN_FLAG			0x01	//Datenbertragung beendet (finished)
#define  SYN_FLAG			0x02	//Verbindungs aufbauen (synchronize)
#define  RST_FLAG			0x04	//Verbindung neu initialisieren (reset)
#define  PSH_FLAG			0x08	//Datenbergabe an Anwendung beschleunigen (push)
#define  ACK_FLAG			0x10	//Datenbertragung besttigen (acknowledged)

//----------------------------------------------------------------------------
//Aufbau eines Ethernetheader
//
//
//
//
struct Ethernet_Header
{
	uint8_t EnetPacketDest[6];	//MAC Zieladresse 6 Byte
	uint8_t EnetPacketSrc[6];	//MAC Quelladresse 6 Byte
	uint16_t EnetPacketType;	//Nutzlast 0x0800=IP Datagramm;0x0806 = ARP
};

//----------------------------------------------------------------------------
//Aufbau eines ARP Header
//	
//	2 BYTE Hardware Typ				|	2 BYTE Protokoll Typ	
//	1 BYTE Lnge Hardwareadresse	|	1 BYTE Lnge Protokolladresse
//	2 BYTE Operation
//	6 BYTE MAC Adresse Absender		|	4 BYTE IP Adresse Absender
//	6 BYTE MAC Adresse Empfnger	|	4 BYTE IP Adresse Empfnger	
struct ARP_Header
{
	uint16_t ARP_HWType;	//Hardware Typ enthlt den Code fr Ethernet oder andere Link Layer
	uint16_t ARP_PRType;	//Protokoll Typ enthlt den Code fr IP o. anderes bertragungsprotokoll
	uint8_t ARP_HWLen;		//Lnge der Hardwareadresse enthlt 6 fr 6 Byte lange MAC Adressen
	uint8_t ARP_PRLen;		//Lnge der Protocolladresse enthlt 4 fr 4 Byte lange IP Adressen
	uint16_t ARP_Op;		//Enthlt Code der signalisiert ob es sich um eine Anfrage o. Antwort handelt
	uint8_t ARP_SHAddr[6];	//Enthlt die MAC Adresse des Anfragenden  
	uint32_t ARP_SIPAddr;   //Enthlt die IP Adresse des Absenders
	uint8_t ARP_THAddr[6];	//MAC Adresse Ziel, ist in diesem Fall 6 * 00,da die Adresse erst noch herausgefunden werden soll (ARP Request)
	uint32_t ARP_TIPAddr;   //IP Adresse enthlt die IP Adresse zu der die Kommunikation aufgebaut werden soll 
};

//----------------------------------------------------------------------------
//Aufbau eines IP Datagramms (B=BIT)
//	
//4B Version	|4B Headergr.	|8B Tos	|16B Gesamtlnge in Bytes	
//16B Identifikation			|3B Schalter	|13B Fragmentierungsposition
//8B Time to Live	|8B Protokoll	|16B Header Prfsumme 
//32B IP Quelladresse
//32B IB Zieladresse
struct IP_Header
{
	uint8_t		IP_Vers_Len;	//4 BIT Die Versionsnummer von IP, 
								//meistens also 4 + 4Bit Headergre 	
	uint8_t		IP_Tos;			//Type of Service
	uint16_t	IP_Pktlen;		//16 Bit Komplette Lng des IP Datagrams in Bytes
	uint16_t	IP_Id;			//ID des Packet fr Fragmentierung oder 
								//Reassemblierung
	uint16_t	IP_Frag_Offset;	//wird benutzt um ein fragmentiertes 
								//IP Packet wieder korrekt zusammenzusetzen
	uint8_t		IP_ttl;			//8 Bit Time to Live die lebenszeit eines Paket
	uint8_t		IP_Proto;		//Zeigt das hherschichtige Protokoll an 
								//(TCP, UDP, ICMP)
	uint16_t	IP_Hdr_Cksum;	//Checksumme des IP Headers
	uint32_t	IP_Srcaddr;		//32 Bit IP Quelladresse
	uint32_t	IP_Destaddr;	//32 Bit IP Zieladresse
};

//----------------------------------------------------------------------------
//Aufbau einer ICMP Nachricht
//	
//8 BIT Typ	|8 BIT Code	|16 BIT Prfsumme	
//Je nach Typ			|Nachricht unterschiedlich
//Testdaten
//
//8 BIT Typ = 0 Echo Reply oder 8 Echo request	
struct ICMP_Header
{
	uint8_t   	ICMP_Type;		//8 bit typ identifiziert Aufgabe der Nachricht 
								//0 = Echo Reply 8 = Echo Request
	uint8_t    	ICMP_Code;		//8 Bit Code enthlt Detailangaben zu bestimmten Typen
	uint16_t   	ICMP_Cksum;		//16 Bit Prfsumme enthlt die CRC Prfsumme
	uint16_t   	ICMP_Id;		//2 byte Identifizierung
	uint16_t   	ICMP_SeqNum;	//Sequenznummer
};

//----------------------------------------------------------------------------
//TCP Header Layout
//
//
struct TCP_Header
{
	unsigned int 	TCP_SrcPort;	//der Quellport fr die Verbindung (Socket)
	unsigned int 	TCP_DestPort;	//der Zielport fr die Verbindung (Socket)
	unsigned long	TCP_Seqnum;	//numerierter Bytestrom
	unsigned long	TCP_Acknum;	//numerierter Bytestrom (Gegenrichtung)
	unsigned char  TCP_Hdrlen;
	unsigned char 	TCP_HdrFlags;//4Bit Lnge des Headers in 32 Bit schritten rest Flags
	unsigned int	TCP_Window;	//Fenstergre max. 64K Byte
	unsigned int	TCP_Chksum;	//Enthlt eine Prfsumme ber Header und Datenbereich
	unsigned int	TCP_UrgentPtr;	//16 Bit fr dringende bertragung
};

void QueryMAC(uint32_t ip);
void InterruptHandler();
// "192" "168" "2" "2"
unsigned long MakeIP(unsigned char byte1, unsigned char byte2, unsigned char byte3, unsigned char byte4);

#endif
